==> open-xchange-usm_7.0.2.orig.tar.bz2: open-xchange-usm-7.0.2/com.openexchange.usm.session.impl/src/com/openexchange/usm/session/sync/OXDataCache.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.usm.session.sync;

import java.lang.ref.SoftReference;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;

import com.openexchange.usm.api.session.DataObject;

/**
 * Cache that globally stores the object data retrieved from the OX server, to be reused if the client executes
 * multiple syncs in a short time for the same folder without executing any changes (e.g. to retrieve multiple
 * small parts of a big synchronization).
 * 
 * @author afe
 *
 */
public class OXDataCache {
	// 5 minutes as interval to check for removal of dead references in the cache table
	private final static long CHECK_INTERVAL = 300000L;

	// Limit in milliseconds how long cached data is considered valid for complete syncs, 0 == caching is completely disabled
	private final long _cacheTimeLimitComplete;

	// Limit in milliseconds how long cached data is considered valid for incomplete syncs, 0 == caching is completely disabled
	private final long _cacheTimeLimitIncomplete;

	private final Map<OXDataCacheID, SoftReference<CachedOXData>> _cache = new ConcurrentHashMap<OXDataCacheID, SoftReference<CachedOXData>>();

	// Used to check periodically for removal of cache entries where the reference has been cleared
	private volatile long _lastRemovalTime = 0L;

	public OXDataCache(long cacheTimeLimitNormal, long cacheTimeLimitIncomplete) {
		_cacheTimeLimitComplete = cacheTimeLimitNormal;
		_cacheTimeLimitIncomplete = cacheTimeLimitIncomplete;
	}

	public CachedOXData getCachedData(OXDataCacheID id) {
		if (!isCacheEnabled())
			return null;
		removeOldReferences();
		SoftReference<CachedOXData> value = _cache.get(id);
		if (value != null) {
			CachedOXData data = value.get();
			if (isCacheDataValid(data, System.currentTimeMillis()))
				return data;
			_cache.remove(id);
		}
		return null;
	}

	public CachedOXData storeCachedData(OXDataCacheID id, DataObject[] data) {
		if (!isCacheEnabled())
			return null;
		removeOldReferences();
		CachedOXData cachedData = new CachedOXData(System.currentTimeMillis(), data);
		_cache.put(id, new SoftReference<CachedOXData>(cachedData));
		return cachedData;
	}

	public void removeCachedData(OXDataCacheID id) {
		if (isCacheEnabled()) {
			removeOldReferences();
			_cache.remove(id);
		}
	}

	private synchronized void removeOldReferences() {
		long now = System.currentTimeMillis();
		synchronized (_cache) {
			if (_lastRemovalTime + CHECK_INTERVAL < now) {
				_lastRemovalTime = now;
				for (Iterator<Entry<OXDataCacheID, SoftReference<CachedOXData>>> i = _cache.entrySet().iterator(); i
						.hasNext();) {
					Entry<OXDataCacheID, SoftReference<CachedOXData>> entry = i.next();
					SoftReference<CachedOXData> val = entry.getValue();
					if (val == null || !isCacheDataValid(val.get(), now))
						i.remove();
				}
			}
		}
	}

	private boolean isCacheDataValid(CachedOXData data, long now) {
		if (data == null)
			return false;
		long limit = data.isSyncComplete() ? _cacheTimeLimitComplete : _cacheTimeLimitIncomplete;
		return limit != 0L && data.getTimestamp() > now - limit;
	}

	private boolean isCacheEnabled() {
		return _cacheTimeLimitComplete != 0L || _cacheTimeLimitIncomplete != 0L;
	}
}

==> open-xchange-usm_7.4.2.orig.tar.bz2: open-xchange-usm-7.4.2/com.openexchange.usm.session.impl/src/com/openexchange/usm/session/sync/OXDataCache.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.usm.session.sync;

import java.lang.ref.SoftReference;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import com.openexchange.usm.api.session.DataObject;

/**
 * Cache that globally stores the object data retrieved from the OX server, to be reused if the client executes
 * multiple syncs in a short time for the same folder without executing any changes (e.g. to retrieve multiple
 * small parts of a big synchronization).
 * 
 * @author afe
 *
 */
public class OXDataCache {
	// 5 minutes as interval to check for removal of dead references in the cache table
	private final static long CHECK_INTERVAL = 300000L;

	// Limit in milliseconds how long cached data is considered valid for complete syncs, 0 == caching is completely disabled
	private final long _cacheTimeLimitComplete;

	// Limit in milliseconds how long cached data is considered valid for incomplete syncs, 0 == caching is completely disabled
	private final long _cacheTimeLimitIncomplete;

	private final Map<OXDataCacheID, SoftReference<CachedOXData>> _cache = new ConcurrentHashMap<OXDataCacheID, SoftReference<CachedOXData>>();

	// Used to check periodically for removal of cache entries where the reference has been cleared
	private volatile long _lastRemovalTime = 0L;

	public OXDataCache(long cacheTimeLimitNormal, long cacheTimeLimitIncomplete) {
		_cacheTimeLimitComplete = cacheTimeLimitNormal;
		_cacheTimeLimitIncomplete = cacheTimeLimitIncomplete;
	}

	public CachedOXData getCachedData(OXDataCacheID id) {
		if (!isCacheEnabled())
			return null;
		removeOldReferences();
		SoftReference<CachedOXData> value = _cache.get(id);
		if (value != null) {
			CachedOXData data = value.get();
			if (isCacheDataValid(data, System.currentTimeMillis()))
				return data;
			_cache.remove(id);
		}
		return null;
	}

	public CachedOXData storeCachedData(OXDataCacheID id, DataObject[] data) {
		if (!isCacheEnabled())
			return null;
		removeOldReferences();
		CachedOXData cachedData = new CachedOXData(System.currentTimeMillis(), data);
		_cache.put(id, new SoftReference<CachedOXData>(cachedData));
		return cachedData;
	}

	public void removeCachedData(OXDataCacheID id) {
		if (isCacheEnabled()) {
			removeOldReferences();
			_cache.remove(id);
		}
	}

	private void removeOldReferences() {
		long now = System.currentTimeMillis();
		synchronized (_cache) {
			if (_lastRemovalTime + CHECK_INTERVAL < now) {
				_lastRemovalTime = now;
				for (Iterator<Entry<OXDataCacheID, SoftReference<CachedOXData>>> i = _cache.entrySet().iterator(); i
						.hasNext();) {
					Entry<OXDataCacheID, SoftReference<CachedOXData>> entry = i.next();
					SoftReference<CachedOXData> val = entry.getValue();
					if (val == null || !isCacheDataValid(val.get(), now))
						i.remove();
				}
			}
		}
	}

	private boolean isCacheDataValid(CachedOXData data, long now) {
		if (data == null)
			return false;
		long limit = data.isSyncComplete() ? _cacheTimeLimitComplete : _cacheTimeLimitIncomplete;
		return limit != 0L && data.getTimestamp() > now - limit;
	}

	private boolean isCacheEnabled() {
		return _cacheTimeLimitComplete != 0L || _cacheTimeLimitIncomplete != 0L;
	}
}
